package com.sample;

import bitronix.tm.BitronixTransactionManager;
import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import javax.naming.InitialContext;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.transaction.TransactionManager;
import javax.transaction.UserTransaction;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.impl.EnvironmentFactory;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.KnowledgeRuntime;
import org.drools.runtime.KnowledgeSessionConfiguration;
import org.drools.runtime.process.WorkItem;
import org.drools.runtime.process.WorkItemHandler;
import org.drools.runtime.process.WorkItemManager;

public class Main
{
	public static void main(String[] args) throws Exception
	{
		PoolingDataSource dataSource = createDataSource();
		BitronixTransactionManager transactionManager = createTransactionManager();

		AtomicInteger counter = new AtomicInteger();
		try
		{
			// Create a session.
			UserTransaction tx = (UserTransaction) new InitialContext().lookup("java:comp/UserTransaction");
			tx.begin();
			KnowledgeRuntime knowledgeRuntime = createKnowledgeRuntime(transactionManager);
			tx.commit();

			// Spawn workers.
			int totalWorkers = 20;
			counter.set(totalWorkers);
			List<Runnable> workers = new ArrayList<Runnable>();
			for (int i = 0; i < totalWorkers; i++)
				workers.add(new ProcessStarter(knowledgeRuntime, counter));

			for (Runnable worker : workers)
				new Thread(worker).start();
		}
		finally
		{
			while (counter.get() != 0)
				Thread.yield();
			cleanup(dataSource, transactionManager);
		}
	}

	private static void cleanup(PoolingDataSource dataSource, BitronixTransactionManager transactionManager)
	{
		transactionManager.shutdown();
		dataSource.close();
	}

	/**
	 * Worker - spawns a new process.
	 */
	private static class ProcessStarter implements Runnable
	{
		private KnowledgeRuntime knowledgeRuntime;

		private AtomicInteger counter = new AtomicInteger();

		private ProcessStarter(KnowledgeRuntime knowledgeRuntime, AtomicInteger counter)
		{
			this.knowledgeRuntime = knowledgeRuntime;
			this.counter = counter;
		}

		public void run()
		{
			try
			{
				UserTransaction tx = (UserTransaction) new InitialContext().lookup("java:comp/UserTransaction");
				tx.begin();
				try
				{
					knowledgeRuntime.startProcess("com.sample.evaluation");
					tx.commit();
				}
				catch (Exception e)
				{
					tx.rollback();
				}
			}
			catch (Exception e)
			{
				throw new RuntimeException(e);
			}
			finally
			{
				counter.decrementAndGet();
			}
		}
	}

	private static KnowledgeRuntime createKnowledgeRuntime(TransactionManager transactionManager)
	{
		KnowledgeSessionConfiguration conf = KnowledgeBaseFactory.newKnowledgeSessionConfiguration();
		Environment env = EnvironmentFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, getEntityManagerFactory());
		env.set(EnvironmentName.TRANSACTION_MANAGER, transactionManager);
		KnowledgeRuntime knowledgeRuntime =
				JPAKnowledgeService.newStatefulKnowledgeSession(getKnowledgeBase(), conf, env);
		registerHumanTaskHandler(knowledgeRuntime);
		return knowledgeRuntime;
	}

	private static BitronixTransactionManager createTransactionManager()
	{
		TransactionManagerServices.getConfiguration().setServerId("example");
		return TransactionManagerServices.getTransactionManager();
	}

	private static EntityManagerFactory getEntityManagerFactory()
	{
		return Persistence.createEntityManagerFactory("org.jbpm");
	}

	private static KnowledgeBase getKnowledgeBase()
	{
		KnowledgeBuilder builder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		builder.add(ResourceFactory.newClassPathResource("HelloProcess.bpmn"), ResourceType.BPMN2);
		return builder.newKnowledgeBase();
	}

	private static PoolingDataSource createDataSource()
	{
		PoolingDataSource dataSource = new PoolingDataSource();
		dataSource.setClassName("org.postgresql.xa.PGXADataSource");
		dataSource.setUniqueName("jdbc/jbpm-ds");
		dataSource.setMaxPoolSize(10);
		dataSource.getDriverProperties().setProperty("user", "jbpm");
		dataSource.getDriverProperties().setProperty("password", "jbpm");
		dataSource.getDriverProperties().setProperty("serverName", "localhost");
		dataSource.getDriverProperties().setProperty("databaseName", "jbpm0");
		dataSource.getDriverProperties().setProperty("portNumber", "5432");
		dataSource.init();
		return dataSource;
	}

	private static void registerHumanTaskHandler(KnowledgeRuntime knowledgeRuntime)
	{
		knowledgeRuntime.getWorkItemManager().registerWorkItemHandler("Human Task", new WorkItemHandler()
		{
			public void executeWorkItem(WorkItem workItem, WorkItemManager workItemManager)
			{
				System.out.println("Executing " + workItem);
			}

			public void abortWorkItem(WorkItem workItem, WorkItemManager workItemManager)
			{
				System.out.println("Aborting " + workItem);
			}
		});
	}
}